@@ -24,7 +24,7 @@
 #include <string.h>
 #include <time.h>
 
-#include <gtk/gtk.h>
+#include <glib.h>
 #include <libgnomevfs/gnome-vfs-types.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 
@@ -33,16 +33,12 @@
 #include "fr-command.h"
 #include "fr-command-zip.h"
 
-
 #define EMPTY_ARCHIVE_WARNING        "zipfile is empty"
 #define EMPTY_ARCHIVE_WARNING_LENGTH 16
 
 static void fr_command_zip_class_init  (FRCommandZipClass *class);
-
-static void fr_command_zip_init        (FRCommand *afile);
-
-static void fr_command_zip_destroy     (GtkObject *object);
-
+static void fr_command_zip_init        (FRCommand         *afile);
+static void fr_command_zip_finalize    (GObject           *object);
 
 /* Parent Class */
 
@@ -52,10 +48,11 @@ static FRCommandClass *parent_class = NULL;
 /* -- list -- */
 
 static time_t
-mktime_from_string (char *date_s, char *time_s)
+mktime_from_string (char *date_s, 
+		    char *time_s)
 {
-	struct tm tm = {0, };
-	char **fields;
+	struct tm   tm = {0, };
+	char      **fields;
 
 	/* date */
 
@@ -64,8 +61,14 @@ mktime_from_string (char *date_s, char *time_s)
 		tm.tm_mon = atoi (fields[0]) - 1;
 		if (fields[1] != NULL) {
 			tm.tm_mday = atoi (fields[1]);
-			if (fields[2] != NULL)
-				tm.tm_year = atoi (fields[2]);
+			if (fields[2] != NULL) { 
+				/* warning : this will work until 2075 ;) */
+				int y = atoi (fields[2]);
+				if (y >= 75)
+					tm.tm_year = atoi (fields[2]);
+				else
+					tm.tm_year = 100 + atoi (fields[2]);
+			}
 		}
 	}
 	g_strfreev (fields);
@@ -97,14 +100,15 @@ eat_spaces (char *line)
 
 
 static char **
-split_line (char *line, int n_fields)
+split_line (char *line, 
+	    int   n_fields)
 {
 	char **fields;
-	char *scan, *field_end;
-	int i;
+	char  *scan, *field_end;
+	int    i;
 
 	fields = g_new0 (char *, n_fields + 1);
-	fields[n_fields + 1] = NULL;
+	fields[n_fields] = NULL;
 
 	scan = eat_spaces (line);
 	for (i = 0; i < n_fields; i++) {
@@ -120,9 +124,9 @@ split_line (char *line, int n_fields)
 static char *
 get_last_field (char *line)
 {
-	int i;
+	int   i;
 	char *field;
-	int n = 8;
+	int   n = 8;
 
 	n--;
 	field = eat_spaces (line);
@@ -136,13 +140,14 @@ get_last_field (char *line)
 
 
 static void
-process_line (char *line, gpointer data)
+list__process_line (char     *line, 
+		    gpointer  data)
 {
-	FileData *fdata;
-	FRCommand *comm = FR_COMMAND (data);
-	char **fields;
-	char *name_field;
-	gint line_l;
+	FileData   *fdata;
+	FRCommand  *comm = FR_COMMAND (data);
+	char      **fields;
+	char       *name_field;
+	gint        line_l;
 
 	g_return_if_fail (line != NULL);
 
@@ -198,6 +203,10 @@ fr_command_zip_list (FRCommand *comm)
 {
 	FR_COMMAND_ZIP (comm)->is_empty = FALSE;
 
+	fr_process_set_out_line_func (FR_COMMAND (comm)->process, 
+				      list__process_line,
+				      comm);
+
 	fr_process_clear (comm->process);
 	fr_process_begin_command (comm->process, "unzip");
 	fr_process_add_arg (comm->process, "-qq");
@@ -205,15 +214,17 @@ fr_command_zip_list (FRCommand *comm)
 	fr_process_add_arg (comm->process, "-l");
 	fr_process_add_arg (comm->process, comm->e_filename);
 	fr_process_end_command (comm->process);
-	fr_process_start (comm->process, TRUE);
+	fr_process_start (comm->process);
 }
 
 
 static void
-fr_command_zip_add (FRCommand *comm,
-		    GList *file_list,
-		    gchar *base_dir,
-		    gboolean update)
+fr_command_zip_add (FRCommand     *comm,
+		    GList         *file_list,
+		    const char    *base_dir,
+		    gboolean       update,
+		    const char    *password,
+		    FRCompression  compression)
 {
 	GList *scan;
 
@@ -228,6 +239,22 @@ fr_command_zip_add (FRCommand *comm,
 	if (update)
 		fr_process_add_arg (comm->process, "-u");
 
+	if (password != NULL) {
+		fr_process_add_arg (comm->process, "-P");
+		fr_process_add_arg (comm->process, password);
+	}
+
+	switch (compression) {
+	case FR_COMPRESSION_VERY_FAST:
+		fr_process_add_arg (comm->process, "-1"); break;
+	case FR_COMPRESSION_FAST:
+		fr_process_add_arg (comm->process, "-3"); break;
+	case FR_COMPRESSION_NORMAL:
+		fr_process_add_arg (comm->process, "-6"); break;
+	case FR_COMPRESSION_MAXIMUM:
+		fr_process_add_arg (comm->process, "-9"); break;
+	}
+
 	fr_process_add_arg (comm->process, comm->e_filename);
 
 	for (scan = file_list; scan; scan = scan->next) 
@@ -239,7 +266,7 @@ fr_command_zip_add (FRCommand *comm,
 
 static void
 fr_command_zip_delete (FRCommand *comm,
-		       GList *file_list)
+		       GList     *file_list)
 {
 	GList *scan;
 
@@ -254,12 +281,13 @@ fr_command_zip_delete (FRCommand *comm,
 
 
 static void
-fr_command_zip_extract (FRCommand *comm,
-			GList *file_list,
-			char *dest_dir,
-			gboolean overwrite,
-			gboolean skip_older,
-			gboolean junk_paths)
+fr_command_zip_extract (FRCommand  *comm,
+			GList      *file_list,
+			const char *dest_dir,
+			gboolean    overwrite,
+			gboolean    skip_older,
+			gboolean    junk_paths,
+			const char *password)
 {
 	GList *scan;
 
@@ -283,6 +311,11 @@ fr_command_zip_extract (FRCommand *comm,
 	if (junk_paths)
 		fr_process_add_arg (comm->process, "-j");
 
+	if (password != NULL) {
+		fr_process_add_arg (comm->process, "-P");
+		fr_process_add_arg (comm->process, password);
+	}
+
 	fr_process_add_arg (comm->process, comm->e_filename);
 
 	for (scan = file_list; scan; scan = scan->next)
@@ -293,7 +326,24 @@ fr_command_zip_extract (FRCommand *comm,
 
 
 static void
-fr_command_zip_handle_error (FRCommand *comm, 
+fr_command_zip_test (FRCommand   *comm,
+		     const char  *password)
+{
+	fr_process_begin_command (comm->process, "unzip");
+	fr_process_add_arg (comm->process, "-t");
+
+	if (password != NULL) {
+		fr_process_add_arg (comm->process, "-P");
+		fr_process_add_arg (comm->process, password);
+	}
+
+	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_end_command (comm->process);
+}
+
+
+static void
+fr_command_zip_handle_error (FRCommand   *comm, 
 			     FRProcError *error)
 {
 	if ((error->type == FR_PROC_ERROR_GENERIC) 
@@ -305,19 +355,19 @@ fr_command_zip_handle_error (FRCommand *comm,
 static void 
 fr_command_zip_class_init (FRCommandZipClass *class)
 {
+        GObjectClass   *gobject_class = G_OBJECT_CLASS (class);
         FRCommandClass *afc;
-        GtkObjectClass *object_class;
 
-        parent_class = gtk_type_class (FR_COMMAND_TYPE);
-        object_class = (GtkObjectClass*) class;
+        parent_class = g_type_class_peek_parent (class);
 	afc = (FRCommandClass*) class;
 
-        object_class->destroy = fr_command_zip_destroy;
+	gobject_class->finalize = fr_command_zip_finalize;
 
         afc->list           = fr_command_zip_list;
 	afc->add            = fr_command_zip_add;
 	afc->delete         = fr_command_zip_delete;
 	afc->extract        = fr_command_zip_extract;
+	afc->test           = fr_command_zip_test;
 	afc->handle_error   = fr_command_zip_handle_error;
 }
 
@@ -330,58 +380,61 @@ fr_command_zip_init (FRCommand *comm)
 	comm->propExtractCanAvoidOverwrite = TRUE;
 	comm->propExtractCanSkipOlder      = TRUE;
 	comm->propExtractCanJunkPaths      = TRUE;
+	comm->propPassword                 = TRUE;
+	comm->propTest                     = TRUE;
 
 	FR_COMMAND_ZIP (comm)->is_empty = FALSE;
 }
 
 
 static void 
-fr_command_zip_destroy (GtkObject *object)
+fr_command_zip_finalize (GObject *object)
 {
         g_return_if_fail (object != NULL);
-        g_return_if_fail (IS_FR_COMMAND_ZIP (object));
+        g_return_if_fail (FR_IS_COMMAND_ZIP (object));
 
-	 /* Chain up */
-        if (GTK_OBJECT_CLASS (parent_class)->destroy)
-                (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+	/* Chain up */
+        if (G_OBJECT_CLASS (parent_class)->finalize)
+		G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 
-GtkType
+GType
 fr_command_zip_get_type ()
 {
-        static guint fr_command_zip_type = 0;
+        static guint type = 0;
 
-        if (!fr_command_zip_type) {
-                GtkTypeInfo fr_command_zip_info = {
-                        "FRCommandZip",
-                        sizeof (FRCommandZip),
-                        sizeof (FRCommandZipClass),
-                        (GtkClassInitFunc) fr_command_zip_class_init,
-                        (GtkObjectInitFunc) fr_command_zip_init,
-                        /* reserved_1 */ NULL,
-                        /* reserved_2 */ NULL,
-                        (GtkClassInitFunc) NULL,
-                };
-                fr_command_zip_type = gtk_type_unique (fr_command_get_type(), 
-						       &fr_command_zip_info);
+        if (! type) {
+                GTypeInfo type_info = {
+			sizeof (FRCommandZipClass),
+			NULL,
+			NULL,
+			(GClassInitFunc) fr_command_zip_class_init,
+			NULL,
+			NULL,
+			sizeof (FRCommandZip),
+			0,
+			(GInstanceInitFunc) fr_command_zip_init
+		};
+
+		type = g_type_register_static (FR_TYPE_COMMAND,
+					       "FRCommandZip",
+					       &type_info,
+					       0);
         }
 
-        return fr_command_zip_type;
+        return type;
 }
 
 
 FRCommand *
-fr_command_zip_new (FRProcess *process,
-		    char *filename)
+fr_command_zip_new (FRProcess  *process,
+		    const char *filename)
 {
 	FRCommand *comm;
 
-	comm = FR_COMMAND (gtk_type_new (fr_command_zip_get_type ()));
+	comm = FR_COMMAND (g_object_new (FR_TYPE_COMMAND_ZIP, NULL));
 	fr_command_construct (comm, process, filename);
-	fr_process_set_proc_line_func (FR_COMMAND (comm)->process, 
-				       process_line,
-				       comm);
 
 	return comm;
 }
